@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: mnemo/data.php,v 1.36.2.8 2007/01/02 13:55:10 jan Exp $
+ * $Horde: mnemo/data.php,v 1.36.2.11 2008/03/10 05:08:10 chuck Exp $
  *
- * Copyright 2001-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL). If you
  * did not receive this file, see http://www.horde.org/licenses/asl.php.
@@ -66,7 +66,6 @@ case 'export':
     $exportID = Util::getFormData('exportID');
 
     /* Create a Mnemo storage instance. */
-    require_once MNEMO_BASE . '/lib/Driver.php';
     $storage = &Mnemo_Driver::singleton(Auth::getAuth());
     $storage->retrieve();
 
@@ -99,6 +98,7 @@ case IMPORT_FILE:
     break;
 }
 
+$next_step = null;
 if (!$error) {
     $data = &Horde_Data::singleton($import_format);
     if (is_a($data, 'PEAR_Error')) {
@@ -121,7 +121,6 @@ if (is_array($next_step)) {
     $categories = $cManager->get();
 
     /* Create a Mnemo storage instance. */
-    require_once MNEMO_BASE . '/lib/Driver.php';
     $storage = &Mnemo_Driver::singleton($_SESSION['import_data']['target']);
     $max_memos = Mnemo::hasPermission('max_notes');
     $num_memos = Mnemo::countMemos();
@@ -208,8 +207,10 @@ $title = _("Import/Export Notes");
 require MNEMO_TEMPLATES . '/common-header.inc';
 require MNEMO_TEMPLATES . '/menu.inc';
 
-foreach ($templates[$next_step] as $template) {
-    require $template;
+if (isset($templates[$next_step])) {
+    foreach ($templates[$next_step] as $template) {
+        require $template;
+    }
 }
 
 require $registry->get('templates', 'horde') . '/common-footer.inc';
